@@ -22,6 +22,7 @@
 #include "oomd/Log.h"
 #include "oomd/OomdContext.h"
 #include "oomd/util/Fixture.h"
+#include "oomd/util/TestHelper.h"
 
 using namespace Oomd;
 using namespace testing;
@@ -31,18 +32,20 @@ class CgroupContextTest : public Test {
  public:
   CgroupContextTest() {
     params_.io_devs = {{"1:10", DeviceType::HDD}, {"1:11", DeviceType::SSD}};
-    params_.hdd_coeffs = {.read_iops = 6,
-                          .readbw = 5,
-                          .write_iops = 4,
-                          .writebw = 3,
-                          .trim_iops = 2,
-                          .trimbw = 1};
-    params_.ssd_coeffs = {.read_iops = 1,
-                          .readbw = 2,
-                          .write_iops = 3,
-                          .writebw = 4,
-                          .trim_iops = 5,
-                          .trimbw = 6};
+    params_.hdd_coeffs = {
+        .read_iops = 6,
+        .readbw = 5,
+        .write_iops = 4,
+        .writebw = 3,
+        .trim_iops = 2,
+        .trimbw = 1};
+    params_.ssd_coeffs = {
+        .read_iops = 1,
+        .readbw = 2,
+        .write_iops = 3,
+        .writebw = 4,
+        .trim_iops = 5,
+        .trimbw = 6};
     ctx_ = OomdContext(params_);
   }
 
@@ -62,7 +65,7 @@ class CgroupContextTest : public Test {
 };
 
 TEST_F(CgroupContextTest, MonitorRootHost) {
-  std::string cgroup2fs_mntpt = Fs::getCgroup2MountPoint();
+  std::string cgroup2fs_mntpt = ASSERT_SYS_OK(Fs::getCgroup2MountPoint());
   if (cgroup2fs_mntpt.empty()) {
 #ifdef GTEST_SKIP
     GTEST_SKIP() << "Host not running cgroup2";
@@ -195,8 +198,8 @@ TEST_F(CgroupContextTest, DistinguishRecreate) {
           {F::makeFile("cgroup.controllers"),
            F::makeFile("memory.current", "123\n")})}));
 
-  CgroupContext cgroup_ctx(ctx_, CgroupPath(tempDir_, "system.slice"));
-  ASSERT_TRUE(cgroup_ctx.fd().isValid());
+  auto cgroup_ctx = ASSERT_EXISTS(
+      CgroupContext::make(ctx_, CgroupPath(tempDir_, "system.slice")));
   ASSERT_EQ(cgroup_ctx.current_usage(), 123);
 
   // Remove cgroup and recreate one with the exact same name
@@ -258,14 +261,16 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
            F::makeFile(
                "memory.stat",
                {"anon 123456789\n"
-                "file 12345678\n"}),
+                "file 12345678\n"
+                "pgscan 4567890123\n"}),
            F::makeFile("memory.swap.current", {"1234\n"}),
+           F::makeFile("memory.swap.max", {"1024\n"}),
            F::makeDir("service1.service", {}),
            F::makeDir("service2.service", {}),
            F::makeDir("service3.service", {})})}));
 
-  CgroupContext cgroup_ctx(ctx_, CgroupPath(tempDir_, "system.slice"));
-  ASSERT_TRUE(cgroup_ctx.fd().isValid());
+  auto cgroup_ctx = ASSERT_EXISTS(
+      CgroupContext::make(ctx_, CgroupPath(tempDir_, "system.slice")));
 
   std::decay_t<decltype(cgroup_ctx.children())> children;
   std::decay_t<decltype(cgroup_ctx.mem_pressure())> mem_pressure;
@@ -274,6 +279,8 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   std::decay_t<decltype(cgroup_ctx.io_stat())> io_stat;
   decltype(cgroup_ctx.current_usage()) current_usage;
   decltype(cgroup_ctx.swap_usage()) swap_usage;
+  decltype(cgroup_ctx.swap_max()) swap_max;
+  decltype(cgroup_ctx.effective_swap_max()) effective_swap_max;
   decltype(cgroup_ctx.memory_low()) memory_low;
   decltype(cgroup_ctx.memory_min()) memory_min;
   decltype(cgroup_ctx.memory_high()) memory_high;
@@ -282,8 +289,10 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   decltype(cgroup_ctx.nr_dying_descendants()) nr_dying_descendants;
   decltype(cgroup_ctx.memory_protection()) memory_protection;
   decltype(cgroup_ctx.io_cost_cumulative()) io_cost_cumulative;
+  decltype(cgroup_ctx.pg_scan_cumulative()) pg_scan_cumulative;
   decltype(cgroup_ctx.average_usage()) average_usage;
   decltype(cgroup_ctx.io_cost_rate()) io_cost_rate;
+  decltype(cgroup_ctx.pg_scan_rate()) pg_scan_rate;
 
   auto set_and_check_fields = [&]() {
     children = cgroup_ctx.children();
@@ -293,6 +302,8 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
     io_stat = cgroup_ctx.io_stat();
     current_usage = cgroup_ctx.current_usage();
     swap_usage = cgroup_ctx.swap_usage();
+    swap_max = cgroup_ctx.swap_max();
+    effective_swap_max = cgroup_ctx.effective_swap_max();
     memory_low = cgroup_ctx.memory_low();
     memory_min = cgroup_ctx.memory_min();
     memory_high = cgroup_ctx.memory_high();
@@ -301,8 +312,10 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
     nr_dying_descendants = cgroup_ctx.nr_dying_descendants();
     memory_protection = cgroup_ctx.memory_protection();
     io_cost_cumulative = cgroup_ctx.io_cost_cumulative();
+    pg_scan_cumulative = cgroup_ctx.pg_scan_cumulative();
     average_usage = cgroup_ctx.average_usage();
     io_cost_rate = cgroup_ctx.io_cost_rate();
+    pg_scan_rate = cgroup_ctx.pg_scan_rate();
 
     ASSERT_TRUE(children);
     ASSERT_TRUE(mem_pressure);
@@ -311,6 +324,8 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
     ASSERT_TRUE(io_stat);
     ASSERT_TRUE(current_usage);
     ASSERT_TRUE(swap_usage);
+    ASSERT_TRUE(swap_max);
+    ASSERT_TRUE(effective_swap_max);
     ASSERT_TRUE(memory_low);
     ASSERT_TRUE(memory_min);
     ASSERT_TRUE(memory_high);
@@ -319,6 +334,7 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
     ASSERT_TRUE(nr_dying_descendants);
     ASSERT_TRUE(memory_protection);
     ASSERT_TRUE(io_cost_cumulative);
+    ASSERT_TRUE(pg_scan_cumulative);
     ASSERT_TRUE(average_usage);
     ASSERT_TRUE(io_cost_rate);
   };
@@ -343,13 +359,18 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   EXPECT_EQ(
       io_pressure->total, std::optional<std::chrono::microseconds>(23456));
   EXPECT_EQ(
-      memory_stat, memory_stat_t({{"anon", 123456789}, {"file", 12345678}}));
+      memory_stat,
+      memory_stat_t(
+          {{"anon", 123456789}, {"file", 12345678}, {"pgscan", 4567890123}}));
   EXPECT_EQ(
       io_stat,
-      IOStat({{"1:10", 1111111, 2222222, 33, 44, 5555555555, 6},
-              {"1:11", 2222222, 3333333, 44, 55, 6666666666, 7}}));
+      IOStat(
+          {{"1:10", 1111111, 2222222, 33, 44, 5555555555, 6},
+           {"1:11", 2222222, 3333333, 44, 55, 6666666666, 7}}));
   EXPECT_EQ(current_usage, 1122334455);
   EXPECT_EQ(swap_usage, 1234);
+  EXPECT_EQ(swap_max, 1024);
+  EXPECT_EQ(effective_swap_max, 0);
   EXPECT_EQ(memory_low, 11223344);
   EXPECT_EQ(memory_min, 112233);
   EXPECT_EQ(memory_high, 2233445566);
@@ -363,6 +384,8 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   // 2222222*2 + 3333333*4 + 44*1 + 55*3 + 6666666666*6 + 7*5
   EXPECT_EQ(io_cost_cumulative, 45585556178);
   EXPECT_EQ(io_cost_rate, 0);
+  EXPECT_EQ(pg_scan_cumulative, 4567890123);
+  EXPECT_EQ(pg_scan_rate, std::nullopt);
 
   // Update most of control files (by adding 1, 0.1 or 0.01)
   F::materialize(F::makeDir(
@@ -398,8 +421,10 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
            F::makeFile(
                "memory.stat",
                {"anon 123456790\n"
-                "file 12345679\n"}),
+                "file 12345679\n"
+                "pgscan 5678901234\n"}),
            F::makeFile("memory.swap.current", {"1235\n"}),
+           F::makeFile("memory.swap.max", {"2048\n"}),
            F::makeDir("service1.service", {}),
            F::makeDir("service2.service", {}),
            F::makeDir("service3.service", {}),
@@ -414,6 +439,8 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   EXPECT_EQ(cgroup_ctx.io_stat(), io_stat);
   EXPECT_EQ(cgroup_ctx.current_usage(), current_usage);
   EXPECT_EQ(cgroup_ctx.swap_usage(), swap_usage);
+  EXPECT_EQ(cgroup_ctx.swap_max(), swap_max);
+  EXPECT_EQ(cgroup_ctx.effective_swap_max(), effective_swap_max);
   EXPECT_EQ(cgroup_ctx.memory_low(), memory_low);
   EXPECT_EQ(cgroup_ctx.memory_min(), memory_min);
   EXPECT_EQ(cgroup_ctx.memory_high(), memory_high);
@@ -424,6 +451,8 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   EXPECT_EQ(cgroup_ctx.average_usage(), average_usage);
   EXPECT_EQ(cgroup_ctx.io_cost_cumulative(), io_cost_cumulative);
   EXPECT_EQ(cgroup_ctx.io_cost_rate(), io_cost_rate);
+  EXPECT_EQ(cgroup_ctx.pg_scan_cumulative(), pg_scan_cumulative);
+  EXPECT_EQ(cgroup_ctx.pg_scan_rate(), pg_scan_rate);
 
   // Call refresh() to clear cache and retrieve values again
   ASSERT_TRUE(cgroup_ctx.refresh());
@@ -445,13 +474,18 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   EXPECT_EQ(
       io_pressure->total, std::optional<std::chrono::microseconds>(23457));
   EXPECT_EQ(
-      memory_stat, memory_stat_t({{"anon", 123456790}, {"file", 12345679}}));
+      memory_stat,
+      memory_stat_t(
+          {{"anon", 123456790}, {"file", 12345679}, {"pgscan", 5678901234}}));
   EXPECT_EQ(
       io_stat,
-      IOStat({{"1:10", 1111112, 2222223, 34, 45, 5555555556, 7},
-              {"1:11", 2222223, 3333334, 45, 56, 6666666667, 8}}));
+      IOStat(
+          {{"1:10", 1111112, 2222223, 34, 45, 5555555556, 7},
+           {"1:11", 2222223, 3333334, 45, 56, 6666666667, 8}}));
   EXPECT_EQ(current_usage, 1122334456);
   EXPECT_EQ(swap_usage, 1235);
+  EXPECT_EQ(swap_max, 2048);
+  EXPECT_EQ(effective_swap_max, 0);
   EXPECT_EQ(memory_low, 11223345);
   EXPECT_EQ(memory_min, 112234);
   EXPECT_EQ(memory_high, 2233445567);
@@ -465,4 +499,152 @@ TEST_F(CgroupContextTest, DataLifeCycle) {
   // 2222223*2 + 3333334*4 + 45*1 + 56*3 + 6666666667*6 + 8*5
   EXPECT_EQ(io_cost_cumulative, 45585556220);
   EXPECT_EQ(io_cost_rate, 45585556220 - 45585556178);
+  EXPECT_EQ(pg_scan_cumulative, 5678901234);
+  EXPECT_EQ(pg_scan_rate, 5678901234 - 4567890123);
+}
+
+TEST_F(CgroupContextTest, EffectiveSwapMax) {
+  ctx_.setSystemContext(SystemContext{.swaptotal = 3000});
+  F::materialize(F::makeDir(
+      tempDir_,
+      {F::makeDir(
+          "system.slice",
+          {F::makeFile("cgroup.controllers"),
+           F::makeFile("memory.swap.max", "1000\n"),
+           F::makeDir(
+               "oomd.service",
+               {F::makeFile("cgroup.controllers"),
+                F::makeFile("memory.swap.max", "2000\n")})})}));
+
+  auto cgroup_ctx = ASSERT_EXISTS(CgroupContext::make(
+      ctx_, CgroupPath(tempDir_, "system.slice/oomd.service")));
+
+  ASSERT_TRUE(cgroup_ctx.effective_swap_max());
+  ASSERT_EQ(*cgroup_ctx.effective_swap_max(), 1000);
+}
+
+TEST_F(CgroupContextTest, EffectiveSwapFree) {
+  ctx_.setSystemContext(SystemContext{.swaptotal = 400, .swapused = 100});
+  F::materialize(F::makeDir(
+      tempDir_,
+      {F::makeDir(
+           "system.slice",
+           {F::makeFile("cgroup.controllers"),
+            F::makeFile("memory.swap.max", "max\n"),
+            F::makeFile("memory.swap.current", "100\n"),
+            F::makeDir(
+                "oomd.service",
+                {
+                    F::makeFile("cgroup.controllers"),
+                    F::makeFile("memory.swap.max", "200\n"),
+                    F::makeFile("memory.swap.current", "100\n"),
+                })}),
+       F::makeDir(
+           "foo.slice",
+           {F::makeFile("cgroup.controllers"),
+            F::makeFile("memory.swap.max", "0\n"),
+            F::makeFile("memory.swap.current", "0\n")})}));
+
+  {
+    // system.slice is limited by the total amount of the swap on the
+    // machine: 400 - 100 = 300
+    auto cgroup_ctx = ASSERT_EXISTS(
+        CgroupContext::make(ctx_, CgroupPath(tempDir_, "system.slice")));
+    ASSERT_TRUE(cgroup_ctx.effective_swap_free());
+    ASSERT_EQ(*cgroup_ctx.effective_swap_free(), 300);
+  }
+  {
+    // oomd.service is limited by memory.swap.max
+    auto cgroup_ctx = ASSERT_EXISTS(CgroupContext::make(
+        ctx_, CgroupPath(tempDir_, "system.slice/oomd.service")));
+    ASSERT_TRUE(cgroup_ctx.effective_swap_free());
+    ASSERT_EQ(*cgroup_ctx.effective_swap_free(), 100);
+  }
+  {
+    // swap.max of 0 on foo.slice results in 0% util
+    auto cgroup_ctx = ASSERT_EXISTS(
+        CgroupContext::make(ctx_, CgroupPath(tempDir_, "foo.slice")));
+    ASSERT_TRUE(cgroup_ctx.effective_swap_free());
+    ASSERT_EQ(*cgroup_ctx.effective_swap_free(), 0);
+  }
+}
+
+TEST_F(CgroupContextTest, EffectiveSwapUtilPct) {
+  ctx_.setSystemContext(SystemContext{.swaptotal = 400, .swapused = 100});
+  F::materialize(F::makeDir(
+      tempDir_,
+      {F::makeDir(
+           "system.slice",
+           {F::makeFile("cgroup.controllers"),
+            F::makeFile("memory.swap.max", "max\n"),
+            F::makeFile("memory.swap.current", "100\n"),
+            F::makeDir(
+                "oomd.service",
+                {
+                    F::makeFile("cgroup.controllers"),
+                    F::makeFile("memory.swap.max", "200\n"),
+                    F::makeFile("memory.swap.current", "100\n"),
+                })}),
+       F::makeDir(
+           "foo.slice",
+           {F::makeFile("cgroup.controllers"),
+            F::makeFile("memory.swap.max", "0\n"),
+            F::makeFile("memory.swap.current", "0\n")})}));
+
+  {
+    // system.slice is limited by the total amount of the swap on the
+    // machine - 100 / 400 = 0.25
+    auto cgroup_ctx = ASSERT_EXISTS(
+        CgroupContext::make(ctx_, CgroupPath(tempDir_, "system.slice")));
+    ASSERT_TRUE(cgroup_ctx.effective_swap_util_pct());
+    ASSERT_EQ(*cgroup_ctx.effective_swap_util_pct(), 0.25);
+  }
+  {
+    // oomd.service is limited by memory.swap.max
+    auto cgroup_ctx = ASSERT_EXISTS(CgroupContext::make(
+        ctx_, CgroupPath(tempDir_, "system.slice/oomd.service")));
+    ASSERT_TRUE(cgroup_ctx.effective_swap_util_pct());
+    ASSERT_EQ(*cgroup_ctx.effective_swap_util_pct(), 0.5);
+  }
+  {
+    // swap.max of 0 on foo.slice results in 0% util
+    auto cgroup_ctx = ASSERT_EXISTS(
+        CgroupContext::make(ctx_, CgroupPath(tempDir_, "foo.slice")));
+    ASSERT_TRUE(cgroup_ctx.effective_swap_util_pct());
+    ASSERT_EQ(*cgroup_ctx.effective_swap_util_pct(), 0);
+  }
+}
+
+TEST_F(CgroupContextTest, EffectiveSwapUtilPctNoSwap) {
+  // Check that with no swap available on the root slice, this still works
+  ctx_.setSystemContext(SystemContext{.swaptotal = 0, .swapused = 0});
+  {
+    auto cgroup_ctx =
+        ASSERT_EXISTS(CgroupContext::make(ctx_, CgroupPath(tempDir_, "")));
+    ASSERT_TRUE(cgroup_ctx.effective_swap_util_pct());
+    ASSERT_EQ(*cgroup_ctx.effective_swap_util_pct(), 0);
+  }
+}
+
+/*
+ * Verify that CgroupContext won't read from a recreated cgroup.
+ */
+TEST_F(CgroupContextTest, MemoryGrowthDoesntDivideByZero) {
+  F::materialize(F::makeDir(
+      tempDir_,
+      {F::makeDir(
+          "system.slice",
+          // Dummy file to make cgroup valid
+          {F::makeFile("cgroup.controllers"),
+           F::makeFile("memory.current", "0\n")})}));
+
+  auto cgroup_ctx = ASSERT_EXISTS(
+      CgroupContext::make(ctx_, CgroupPath(tempDir_, "system.slice")));
+
+  ASSERT_TRUE(cgroup_ctx.average_usage());
+  ASSERT_EQ(*cgroup_ctx.average_usage(), 0);
+
+  ASSERT_NO_THROW(cgroup_ctx.memory_growth());
+  ASSERT_TRUE(cgroup_ctx.memory_growth());
+  ASSERT_EQ(*cgroup_ctx.memory_growth(), 0);
 }
