@@ -91,6 +91,11 @@ class LogStream {
     ENABLE,
   };
 
+  class Offset {
+   public:
+    uint64_t n;
+  };
+
   LogStream();
   explicit LogStream(LogBase& sink);
   ~LogStream();
@@ -136,12 +141,23 @@ class LogStream {
 template <>
 LogStream& LogStream::operator<<<LogStream::Control>(const Control& ctrl);
 
+template <>
+LogStream& LogStream::operator<<<LogStream::Offset>(const Offset& offset);
+
 template <typename... Args>
 static void OOMD_KMSG_LOG(Args&&... args) {
   Log::get().kmsgLog(std::forward<Args>(args)...);
 }
 
+#ifdef __FILE_NAME__
+#define FILENAME __FILE_NAME__
+#elif defined __BASE_NAME__
+#define FILENAME __BASE_NAME__
+#else
+#define FILENAME __FILE__
+#endif
+
 // This has to be a macro so __FILE__ and __LINE__ are captured
-#define OLOG ::Oomd::LogStream() << "[" << __FILE__ << ":" << __LINE__ << "] "
+#define OLOG ::Oomd::LogStream() << "[" << FILENAME << ":" << __LINE__ << "] "
 
 } // namespace Oomd
