@@ -17,9 +17,11 @@
 
 #pragma once
 
+#include <sys/types.h>
 #include <algorithm>
 #include <functional>
 #include <memory>
+#include <optional>
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
@@ -31,9 +33,18 @@
 
 namespace Oomd {
 
+namespace Engine {
+class Ruleset;
+class PrekillHook;
+class PrekillHookInvocation;
+} // namespace Engine
+
 struct ActionContext {
-  std::string ruleset;
+  std::string ruleset_name;
   std::string detectorgroup;
+  std::string action_group_run_uuid;
+  std::optional<std::chrono::steady_clock::time_point> prekill_hook_timeout_ts{
+      std::nullopt};
 };
 
 struct ContextParams {
@@ -78,6 +89,19 @@ class OomdContext {
   std::vector<ConstCgroupContextRef> addToCacheAndGet(
       const std::unordered_set<CgroupPath>& cgroups);
 
+  /*
+   * Get child of cgroup, adding it to the cache if it doesn't exist yet.
+   */
+  std::optional<ConstCgroupContextRef> addChildToCacheAndGet(
+      const CgroupContext& cgroup_ctx,
+      const std::string& child);
+
+  /*
+   * Get children of cgroup, adding them to the cache if they don't exist yet.
+   */
+  std::vector<ConstCgroupContextRef> addChildrenToCacheAndGet(
+      const CgroupContext& cgroup_ctx);
+
   /*
    * Add a set of cgroups to cache, and return the resulting CgroupContext
    * sorting in descending order by the get_key functor, which accepts a const
@@ -94,6 +118,26 @@ class OomdContext {
     return sorted;
   }
 
+  /*
+   * Sorts cgroups by kill_preference, then get_key. Highest first to lowest
+   * last. Returns new vec; does not mutate cgroups arg.
+   */
+  template <class Functor>
+  static std::vector<ConstCgroupContextRef> sortDescWithKillPrefs(
+      const std::vector<ConstCgroupContextRef>& cgroups,
+      Functor&& get_key) {
+    auto sorted = cgroups;
+    std::sort(sorted.begin(), sorted.end(), [&](const auto& a, const auto& b) {
+      return std::make_tuple(
+                 a.get().kill_preference().value_or(KillPreference::NORMAL),
+                 get_key(a.get())) >
+          std::make_tuple(
+                 b.get().kill_preference().value_or(KillPreference::NORMAL),
+                 get_key(b.get()));
+    });
+    return sorted;
+  }
+
   /*
    * Dumps OomdContext state to stderr
    */
@@ -115,6 +159,29 @@ class OomdContext {
   const SystemContext& getSystemContext() const;
   void setSystemContext(const SystemContext& context);
 
+  /*
+   * Used for plugins to know how long it's been since their last run()
+   */
+  uint64_t getCurrentTick();
+  void bumpCurrentTick();
+
+  /*
+   * Lets action plugins call pause_actions(post_action_delay) on their owning
+   * ruleset. Only available while the ruleset is running its action chain.
+   */
+  const std::optional<Engine::Ruleset*> getInvokingRuleset();
+  void setInvokingRuleset(std::optional<Engine::Ruleset*> ruleset);
+
+  /*
+   * Used to let kill plugins invoke prekill hooks
+   */
+  std::optional<std::unique_ptr<Engine::PrekillHookInvocation>> firePrekillHook(
+      const CgroupContext& cgroup_ctx);
+  void setPrekillHooksHandler(
+      std::function<
+          std::optional<std::unique_ptr<Engine::PrekillHookInvocation>>(
+              const CgroupContext& cgroup_ctx)> prekill_hook_handler);
+
   /*
    * Refresh all cgroups and remove ones no longer exist.
    */
@@ -128,6 +195,11 @@ class OomdContext {
   std::unordered_map<CgroupPath, CgroupContext> cgroups_;
   ActionContext action_context_;
   SystemContext system_ctx_;
+  uint64_t current_tick_{0};
+  std::optional<Engine::Ruleset*> invoking_ruleset_{std::nullopt};
+  std::function<std::optional<std::unique_ptr<Engine::PrekillHookInvocation>>(
+      const CgroupContext& cgroup_ctx)>
+      prekill_hook_handler_{nullptr};
 };
 
 } // namespace Oomd
