@@ -24,4 +24,9 @@ PluginRegistry<Engine::BasePlugin>& getPluginRegistry() {
   return r;
 }
 
+PluginRegistry<Engine::PrekillHook>& getPrekillHookRegistry() {
+  static PluginRegistry<Engine::PrekillHook> r;
+  return r;
+}
+
 } // namespace Oomd
