@@ -27,11 +27,11 @@
 #include <chrono>
 #include <cstring>
 #include <iostream>
+#include <optional>
 
 #include "oomd/Stats.h"
 #include "oomd/StatsClient.h"
 #include "oomd/include/Assert.h"
-#include "oomd/util/Fs.h"
 #include "oomd/util/ScopeGuard.h"
 #include "oomd/util/Util.h"
 
