@@ -16,6 +16,7 @@
 
 #include "oomd/config/ConfigCompiler.h"
 
+#include <optional>
 #include <vector>
 
 #include "oomd/Log.h"
@@ -28,8 +29,9 @@
 
 namespace {
 
-template <typename T>
-std::unique_ptr<Oomd::Engine::BasePlugin> compilePlugin(
+template <typename T, typename PluginT>
+std::unique_ptr<PluginT> compilePluginGeneric(
+    Oomd::PluginRegistry<PluginT>& registry,
     const T& plugin,
     const Oomd::PluginConstructionContext& context) {
   if (plugin.name.empty()) {
@@ -37,8 +39,7 @@ std::unique_ptr<Oomd::Engine::BasePlugin> compilePlugin(
     return nullptr;
   }
 
-  std::unique_ptr<Oomd::Engine::BasePlugin> instance(
-      Oomd::getPluginRegistry().create(plugin.name));
+  std::unique_ptr<PluginT> instance(registry.create(plugin.name));
   if (!instance) {
     OLOG << "Could not locate plugin=" << plugin.name << " in plugin registry";
     return nullptr;
@@ -55,6 +56,22 @@ std::unique_ptr<Oomd::Engine::BasePlugin> compilePlugin(
   return instance;
 }
 
+template <typename T>
+std::unique_ptr<Oomd::Engine::BasePlugin> compilePlugin(
+    const T& plugin,
+    const Oomd::PluginConstructionContext& context) {
+  return compilePluginGeneric<T, Oomd::Engine::BasePlugin>(
+      Oomd::getPluginRegistry(), plugin, context);
+}
+
+std::unique_ptr<Oomd::Engine::PrekillHook> compilePrekillHook(
+    const Oomd::Config2::IR::PrekillHook& hook,
+    const Oomd::PluginConstructionContext& context) {
+  return compilePluginGeneric<
+      Oomd::Config2::IR::PrekillHook,
+      Oomd::Engine::PrekillHook>(Oomd::getPrekillHookRegistry(), hook, context);
+}
+
 std::unique_ptr<Oomd::Engine::DetectorGroup> compileDetectorGroup(
     const Oomd::Config2::IR::DetectorGroup& group,
     const Oomd::PluginConstructionContext& context) {
@@ -89,6 +106,9 @@ std::unique_ptr<Oomd::Engine::Ruleset> compileRuleset(
     bool dropin,
     const Oomd::PluginConstructionContext& context) {
   uint32_t silenced_logs = 0;
+  int post_action_delay = DEFAULT_POST_ACTION_DELAY;
+  int prekill_hook_timeout = DEFAULT_PREKILL_HOOK_TIMEOUT;
+
   std::vector<std::unique_ptr<Oomd::Engine::DetectorGroup>> detector_groups;
   std::vector<std::unique_ptr<Oomd::Engine::BasePlugin>> actions;
 
@@ -122,6 +142,24 @@ std::unique_ptr<Oomd::Engine::Ruleset> compileRuleset(
     return nullptr;
   }
 
+  // post_action_delay field is optional
+  if (ruleset.post_action_delay.size()) {
+    post_action_delay = std::stoi(ruleset.post_action_delay);
+    if (post_action_delay < 0) {
+      OLOG << "Ruleset post_action_delay must be non-negative";
+      return nullptr;
+    }
+  }
+
+  // prekill_hook_timeout field is optional
+  if (ruleset.prekill_hook_timeout.size()) {
+    prekill_hook_timeout = std::stoi(ruleset.prekill_hook_timeout);
+    if (prekill_hook_timeout < 0) {
+      OLOG << "Ruleset prekill_hook_timeout must be non-negative";
+      return nullptr;
+    }
+  }
+
   for (const auto& dg : ruleset.dgs) {
     auto compiled_detectorgroup = compileDetectorGroup(dg, context);
     if (!compiled_detectorgroup) {
@@ -148,7 +186,9 @@ std::unique_ptr<Oomd::Engine::Ruleset> compileRuleset(
       ruleset.dropin.disable_on_drop_in,
       ruleset.dropin.detectorgroups_enabled,
       ruleset.dropin.actiongroup_enabled,
-      silenced_logs);
+      silenced_logs,
+      post_action_delay,
+      prekill_hook_timeout);
 }
 
 } // namespace
@@ -170,14 +210,27 @@ std::unique_ptr<Engine::Engine> compile(
     rulesets.emplace_back(std::move(compiled_ruleset));
   }
 
-  return std::make_unique<Engine::Engine>(std::move(rulesets));
+  std::vector<std::unique_ptr<Engine::PrekillHook>> prekill_hooks;
+
+  for (const auto& prekill_hook : root.prekill_hooks) {
+    auto compiled_prekill_hook_plugin =
+        compilePrekillHook(prekill_hook, context);
+    if (!compiled_prekill_hook_plugin) {
+      return nullptr;
+    }
+
+    prekill_hooks.emplace_back(std::move(compiled_prekill_hook_plugin));
+  }
+
+  return std::make_unique<Engine::Engine>(
+      std::move(rulesets), std::move(prekill_hooks));
 }
 
-std::optional<DropInUnit> compileDropIn(
+std::optional<Engine::DropInUnit> compileDropIn(
     const IR::Root& root,
     const IR::Root& dropin,
     const PluginConstructionContext& context) {
-  DropInUnit ret;
+  Engine::DropInUnit ret;
 
   for (const auto& dropin_rs : dropin.rulesets) {
     bool found_target = false;
@@ -212,6 +265,17 @@ std::optional<DropInUnit> compileDropIn(
     }
   }
 
+  for (const auto& prekill_hook : dropin.prekill_hooks) {
+    auto compiled_prekill_hook_plugin =
+        compilePrekillHook(prekill_hook, context);
+    if (!compiled_prekill_hook_plugin) {
+      OLOG << "Unknown PrekillHook \"" << prekill_hook.name << "\"";
+      return std::nullopt;
+    }
+
+    ret.prekill_hooks.emplace_back(std::move(compiled_prekill_hook_plugin));
+  }
+
   return ret;
 }
 
