@@ -46,6 +46,8 @@ TEST(JsonConfigParserTest, LoadIR) {
   EXPECT_EQ(first.name, "my first ruleset");
   ASSERT_EQ(first.dgs.size(), 1);
   EXPECT_EQ(first.silence_logs, "engine,plugins");
+  EXPECT_EQ(first.post_action_delay, "10");
+  EXPECT_EQ(first.prekill_hook_timeout, "40");
 
   // Check first ruleset drop in config
   EXPECT_TRUE(first.dropin.detectorgroups_enabled);
@@ -92,6 +94,11 @@ TEST(JsonConfigParserTest, LoadIR) {
   EXPECT_EQ(second.acts[2].name, "kill_by_swap_usage");
   EXPECT_EQ(
       second.acts[2].args.at("cgroup"), "workload.slice/workload-tw.slice");
+
+  // Check prekill hooks
+  ASSERT_EQ(root->prekill_hooks.size(), 1);
+  ASSERT_EQ(root->prekill_hooks[0].name, "hypothetical_prekill_hook");
+  ASSERT_EQ(root->prekill_hooks[0].args.size(), 0);
 }
 
 TEST(JsonConfigParserTest, LoadIRBadInput) {
