@@ -25,6 +25,7 @@
 #include "oomd/PluginConstructionContext.h"
 #include "oomd/include/CgroupPath.h"
 #include "oomd/include/Types.h"
+#include "oomd/util/PluginArgParser.h"
 
 namespace Oomd {
 namespace Engine {
@@ -32,10 +33,9 @@ namespace Engine {
 enum class PluginRet {
   CONTINUE = 0,
   STOP,
+  ASYNC_PAUSED,
 };
 
-using PluginArgs = std::unordered_map<std::string, std::string>;
-
 class BasePlugin {
  public:
   /*
@@ -78,6 +78,7 @@ class BasePlugin {
 
   virtual void setName(const std::string& name) {
     name_ = name;
+    argParser_.setName(name_);
   }
   virtual const std::string& getName() const {
     return name_;
@@ -85,6 +86,9 @@ class BasePlugin {
 
   virtual ~BasePlugin() = default;
 
+ protected:
+  PluginArgParser argParser_;
+
  private:
   std::string name_;
 };
