@@ -24,14 +24,22 @@
 
 #include "oomd/OomdContext.h"
 #include "oomd/engine/BasePlugin.h"
+#include "oomd/engine/PrekillHook.h"
 #include "oomd/engine/Ruleset.h"
 
 namespace Oomd {
 namespace Engine {
 
+struct DropInUnit {
+  std::vector<std::unique_ptr<PrekillHook>> prekill_hooks;
+  std::vector<std::unique_ptr<Ruleset>> rulesets;
+};
+
 class Engine {
  public:
-  explicit Engine(std::vector<std::unique_ptr<Ruleset>> rulesets);
+  explicit Engine(
+      std::vector<std::unique_ptr<Ruleset>> rulesets,
+      std::vector<std::unique_ptr<PrekillHook>> prekill_hooks);
   ~Engine() = default;
 
   /*
@@ -42,12 +50,16 @@ class Engine {
    *
    * @returns false if @param ruleset's target is not found. true otherwise.
    */
-  bool addDropInConfig(size_t tag, std::unique_ptr<Ruleset> ruleset);
+  bool addDropInConfig(const std::string& tag, DropInUnit unit);
+
+  bool addDropInRuleset(
+      const std::string& tag,
+      std::unique_ptr<Ruleset> ruleset);
 
   /*
    * Removes drop in configs associated with @param tag
    */
-  void removeDropInConfig(size_t tag);
+  void removeDropInConfig(const std::string& tag);
 
   /*
    * Preruns every @class Ruleset once.
@@ -59,9 +71,13 @@ class Engine {
    */
   void runOnce(OomdContext& context);
 
+  std::optional<std::unique_ptr<PrekillHookInvocation>> firePrekillHook(
+      const CgroupContext& cgroup_ctx,
+      const OomdContext& oomd_context);
+
  private:
   struct DropInRuleset {
-    size_t tag{0}; // required field
+    std::string tag; // required field
     std::unique_ptr<Ruleset> ruleset;
   };
 
@@ -71,6 +87,15 @@ class Engine {
   };
 
   std::vector<BaseRuleset> rulesets_;
+
+  struct TaggedPrekillHook {
+    // dropin_tag is nullopt if hook is not a dropin
+    std::optional<std::string> dropin_tag;
+    std::unique_ptr<PrekillHook> hook;
+  };
+  // stored in reverse order so that dropins, which are pushed to the back,
+  // are run first
+  std::vector<TaggedPrekillHook> prekill_hooks_in_reverse_order_;
 };
 
 } // namespace Engine
