@@ -18,6 +18,7 @@
 #include "oomd/include/CgroupPath.h"
 
 #include <exception>
+#include <optional>
 
 #include "oomd/util/Fs.h"
 #include "oomd/util/Util.h"
@@ -80,7 +81,12 @@ CgroupPath CgroupPath::getChild(const std::string& path) const {
 
 std::vector<CgroupPath> CgroupPath::resolveWildcard() const {
   std::vector<CgroupPath> ret;
-  for (const auto& path : Fs::glob(absolutePath(), /* dir_only */ true)) {
+  auto glob = Fs::glob(absolutePath(), /* dir_only */ true);
+  // TODO(dschatzberg): Report error
+  if (!glob) {
+    return ret;
+  }
+  for (const auto& path : *glob) {
     if (path.find(cgroup_fs_) == 0) {
       if (path.size() == cgroup_fs_.size()) {
         ret.emplace_back(cgroup_fs_, "");
@@ -92,6 +98,19 @@ std::vector<CgroupPath> CgroupPath::resolveWildcard() const {
   return ret;
 }
 
+bool CgroupPath::hasDescendantWithPrefixMatching(
+    const CgroupPath& pattern) const {
+  unsigned int prefix_len =
+      std::min(cgroup_path_.size(), pattern.cgroup_path_.size());
+  for (unsigned int i = 0; i < prefix_len; i++) {
+    if (!(cgroup_path_[i] == pattern.cgroup_path_[i] ||
+          pattern.cgroup_path_[i] == "*")) {
+      return false;
+    }
+  }
+  return true;
+}
+
 bool CgroupPath::operator==(const CgroupPath& other) const {
   return this->absolutePath() == other.absolutePath();
 }
