@@ -18,6 +18,7 @@
 #pragma once
 
 #include "oomd/engine/BasePlugin.h"
+#include "oomd/util/SystemMaybe.h"
 
 #include <chrono>
 #include <map>
@@ -61,15 +62,39 @@ class Senpai : public Engine::BasePlugin {
     std::chrono::microseconds cumulative{0};
     // Count-down to decision to probe/backoff
     int64_t ticks;
+    // Probe statistics for logging
+    uint64_t probe_bytes{0};
+    uint64_t probe_count{0};
   };
 
+  std::optional<bool> hasMemoryReclaim(const CgroupContext& cgroup_ctx);
   std::optional<bool> hasMemoryHighTmp(const CgroupContext& cgroup_ctx);
   std::optional<int64_t> readMemhigh(const CgroupContext& cgroup_ctx);
   bool writeMemhigh(const CgroupContext& cgroup_ctx, int64_t value);
+  bool writeMemhighTimeout(
+      const CgroupContext& cgroup_ctx,
+      int64_t value,
+      std::chrono::milliseconds timeout);
+  bool resetMemhigh(const CgroupContext& cgroup_ctx);
+  bool reclaim(const CgroupContext& cgroup_ctx, int64_t size);
+  SystemMaybe<int64_t> getReclaimableBytes(const CgroupContext& cgroup_ctx);
+  std::optional<int64_t> getLimitMinBytes(const CgroupContext& cgroup_ctx);
+  std::optional<int64_t> getLimitMaxBytes(const CgroupContext& cgroup_ctx);
+  void checkAndLogHighPressure(const CgroupContext& cgroup_ctx) const;
+  SystemMaybe<bool> validatePressure(const CgroupContext& cgroup_ctx) const;
+  SystemMaybe<bool> validateSwap(const CgroupContext& cgroup_ctx) const;
+  SystemMaybe<double> calculateSwappinessFactor(
+      const CgroupContext& cgroup_ctx) const;
 
   bool tick(const CgroupContext& cgroup_ctx, CgroupState& state);
+  bool tick_immediate_backoff(
+      const CgroupContext& cgroup_ctx,
+      CgroupState& state);
   std::optional<CgroupState> initializeCgroup(const CgroupContext& cgroup_ctx);
 
+  int64_t host_mem_total_{0};
+
+  std::optional<bool> has_memory_reclaim_{};
   std::optional<bool> has_memory_high_tmp_{};
 
   std::unordered_set<CgroupPath> cgroups_;
@@ -77,10 +102,16 @@ class Senpai : public Engine::BasePlugin {
 
   // cgroup size limits
   int64_t limit_min_bytes_{100ull << 20};
-  int64_t limit_max_bytes_{500ull << 30};
+  int64_t limit_max_bytes_{10ull << 30};
   // pressure target - stall time over sampling period
   int64_t interval_{6};
-  std::chrono::microseconds pressure_ms_{std::chrono::milliseconds{10}};
+  // interval between aggregation logging; only for immediate_backoff
+  int64_t log_interval_{60};
+  int64_t log_ticks_{0};
+  std::chrono::milliseconds pressure_ms_{10};
+  // Currently only used for immediate backoff
+  double mem_pressure_pct_{0.1};
+  double io_pressure_pct_{0.1};
   // translate observed target deviation to cgroup adjustment rate
   // - max_probe is reached when stalling falls below pressure / coeff_probe
   // - max_backoff is reached when stalling exceeds pressure * coeff_backoff
@@ -88,6 +119,12 @@ class Senpai : public Engine::BasePlugin {
   double max_backoff_{1.0};
   double coeff_probe_{10};
   double coeff_backoff_{20};
+  double swap_threshold_{0.8};
+  int64_t swapout_bps_threshold_{1ull << 20};
+  std::chrono::milliseconds memory_high_timeout_{};
+  bool swap_validation_{false};
+  bool immediate_backoff_{false};
+  bool modulate_swappiness_{false};
 };
 
 } // namespace Oomd
