@@ -15,6 +15,8 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <optional>
+
 #include "oomd/util/Fs.h"
 
 #include <dirent.h>
@@ -33,6 +35,7 @@
 #include <utility>
 
 #include "oomd/include/Assert.h"
+#include "oomd/util/ScopeGuard.h"
 #include "oomd/util/Util.h"
 
 namespace {
@@ -63,33 +66,54 @@ PsiFormat getPsiFormat(const std::vector<std::string>& lines) {
 
 namespace Oomd {
 
-Fs::Fd Fs::Fd::open(const std::string& path, bool read_only) {
-  int flags = read_only ? O_RDONLY : O_WRONLY;
-  return Fd(::open(path.c_str(), flags));
-}
-
-Fs::Fd
+SystemMaybe<Fs::Fd>
 Fs::Fd::openat(const DirFd& dirfd, const std::string& path, bool read_only) {
   int flags = read_only ? O_RDONLY : O_WRONLY;
-  return Fd(::openat(dirfd.fd(), path.c_str(), flags));
+  const auto fd = ::openat(dirfd.fd(), path.c_str(), flags);
+  if (fd == -1) {
+    return SYSTEM_ERROR(errno);
+  }
+  return Fd(fd);
 }
 
-std::optional<uint64_t> Fs::Fd::inode() const {
+SystemMaybe<Fs::Fd> Fs::Fd::open(const std::string& path, bool read_only) {
+  int flags = read_only ? O_RDONLY : O_WRONLY;
+  const auto fd = ::open(path.c_str(), flags);
+  if (fd == -1) {
+    return SYSTEM_ERROR(errno);
+  }
+  return Fd(fd);
+}
+
+SystemMaybe<uint64_t> Fs::Fd::inode() const {
   struct ::stat buf;
   if (::fstat(fd_, &buf) == 0) {
     return static_cast<uint64_t>(buf.st_ino);
   }
-  return std::nullopt;
+  return SYSTEM_ERROR(errno);
 }
 
-void Fs::Fd::close() const {
-  if (isValid()) {
+void Fs::Fd::close() {
+  if (fd_ != -1) {
     ::close(fd_);
+    fd_ = -1;
   }
 }
 
-Fs::DirFd Fs::DirFd::open(const std::string& path) {
-  return DirFd(::open(path.c_str(), O_RDONLY | O_DIRECTORY));
+SystemMaybe<Fs::DirFd> Fs::DirFd::open(const std::string& path) {
+  int fd = ::open(path.c_str(), O_RDONLY | O_DIRECTORY);
+  if (fd == -1) {
+    return SYSTEM_ERROR(errno);
+  }
+  return DirFd(fd);
+}
+
+SystemMaybe<Fs::DirFd> Fs::DirFd::openChildDir(const std::string& path) const {
+  int child_fd = ::openat(fd(), path.c_str(), O_RDONLY | O_DIRECTORY);
+  if (child_fd == -1) {
+    return SYSTEM_ERROR(errno);
+  }
+  return DirFd(child_fd);
 }
 
 bool Fs::isCgroupValid(const DirFd& dirfd) {
@@ -97,14 +121,8 @@ bool Fs::isCgroupValid(const DirFd& dirfd) {
   return ::faccessat(dirfd.fd(), kControllersFile, F_OK, 0) == 0;
 }
 
-struct Fs::DirEnts Fs::readDir(const std::string& path, int flags) {
-  DIR* d;
-  struct Fs::DirEnts de;
-
-  d = ::opendir(path.c_str());
-  if (!d) {
-    return de;
-  }
+SystemMaybe<Fs::DirEnts> Fs::readDirFromDIR(DIR* d, int flags) {
+  Fs::DirEnts de;
 
   while (struct dirent* dir = ::readdir(d)) {
     if (dir->d_name[0] == '.') {
@@ -126,11 +144,10 @@ struct Fs::DirEnts Fs::readDir(const std::string& path, int flags) {
       continue;
     }
 
-    auto file = path + "/" + dir->d_name;
     struct stat buf;
-    int ret = ::lstat(file.c_str(), &buf);
+    int ret = ::fstatat(dirfd(d), dir->d_name, &buf, AT_SYMLINK_NOFOLLOW);
     if (ret == -1) {
-      continue;
+      return SYSTEM_ERROR(errno);
     }
 
     if ((flags & DirEntFlags::DE_FILE) && (buf.st_mode & S_IFREG)) {
@@ -141,10 +158,43 @@ struct Fs::DirEnts Fs::readDir(const std::string& path, int flags) {
     }
   }
 
-  ::closedir(d);
   return de;
 }
 
+SystemMaybe<Fs::DirEnts> Fs::readDirAt(const DirFd& dirfd, int flags) {
+  // once an fd is passed to fdopendir, it is unusable except through that DIR
+  int fresh_fd = ::dup(dirfd.fd());
+
+  DIR* d = ::fdopendir(fresh_fd);
+  if (!d) {
+    auto e = errno;
+    ::close(fresh_fd);
+    return SYSTEM_ERROR(e);
+  }
+
+  OOMD_SCOPE_EXIT {
+    // even though we dup()ed dirfd.fd(), the copied fd shares state with
+    // dirfd.fd(). For dirfd to be reusable again, we need to rewind it, which
+    // we can do through d.
+    ::rewinddir(d);
+
+    ::closedir(d);
+  };
+
+  return Fs::readDirFromDIR(d, flags);
+}
+
+SystemMaybe<Fs::DirEnts> Fs::readDir(const std::string& path, int flags) {
+  DIR* d = ::opendir(path.c_str());
+  if (!d) {
+    return SYSTEM_ERROR(errno);
+  }
+  OOMD_SCOPE_EXIT {
+    ::closedir(d);
+  };
+  return readDirFromDIR(d, flags);
+}
+
 bool Fs::isDir(const std::string& path) {
   struct stat sb;
   if (!::stat(path.c_str(), &sb) && S_ISDIR(sb.st_mode)) {
@@ -154,14 +204,20 @@ bool Fs::isDir(const std::string& path) {
   return false;
 }
 
-std::vector<std::string> Fs::glob(const std::string& pattern, bool dir_only) {
+SystemMaybe<std::vector<std::string>> Fs::glob(
+    const std::string& pattern,
+    bool dir_only) {
   glob_t globbuf;
   std::vector<std::string> ret;
-  int flags = GLOB_NOSORT | GLOB_BRACE;
+  int flags = GLOB_NOSORT | GLOB_BRACE | GLOB_ERR;
   if (dir_only) {
     flags |= GLOB_ONLYDIR;
   }
-  if (0 == ::glob(pattern.c_str(), flags, nullptr, &globbuf)) {
+  auto ec = ::glob(pattern.c_str(), flags, nullptr, &globbuf);
+  OOMD_SCOPE_EXIT {
+    ::globfree(&globbuf);
+  };
+  if (!ec) {
     for (size_t i = 0; i < globbuf.gl_pathc; i++) {
       std::string path(globbuf.gl_pathv[i]);
       // GLOB_ONLYDIR is a hint and we need to double check
@@ -170,9 +226,18 @@ std::vector<std::string> Fs::glob(const std::string& pattern, bool dir_only) {
       }
       ret.emplace_back(std::move(path));
     }
+    return ret;
+  } else if (ec == GLOB_NOMATCH) {
+    // Error with GLOB_NOMATCH just means nothing matched, return empty vec
+    return ret;
+  } else if (ec == GLOB_NOSPACE) {
+    return SYSTEM_ERROR(ENOMEM);
+  } else if (ec == GLOB_ABORTED) {
+    return SYSTEM_ERROR(EINVAL);
+  } else {
+    // Shouldn't be possible, ENOSYS seems reasonable
+    return SYSTEM_ERROR(ENOSYS);
   }
-  ::globfree(&globbuf);
-  return ret;
 }
 
 void Fs::removePrefix(std::string& str, const std::string& prefix) {
@@ -188,33 +253,43 @@ void Fs::removePrefix(std::string& str, const std::string& prefix) {
 }
 
 /* Reads a file and returns a newline separated vector of strings */
-std::vector<std::string> Fs::readFileByLine(const std::string& path) {
+SystemMaybe<std::vector<std::string>> Fs::readFileByLine(
+    const std::string& path,
+    const char delim) {
   std::ifstream f(path, std::ios::in);
   if (!f.is_open()) {
-    return {};
+    return SYSTEM_ERROR(ENOENT);
   }
 
   std::string s;
   std::vector<std::string> v;
-  while (std::getline(f, s)) {
+  while (std::getline(f, s, f.widen(delim))) {
     v.push_back(std::move(s));
   }
 
+  // Error when reading file, and thus content might be corrupted
+  if (f.bad()) {
+    return SYSTEM_ERROR(EINVAL);
+  }
+
   return v;
 }
 
-std::vector<std::string> Fs::readFileByLine(Fd&& fd) {
-  if (!fd.isValid()) {
-    return {};
-  }
-  auto fp = fdopen(fd.fd(), "r");
+SystemMaybe<std::vector<std::string>> Fs::readFileByLine(Fd&& fd) {
+  auto fp = ::fdopen(std::move(fd).fd(), "r");
   if (fp == nullptr) {
-    return {};
+    return SYSTEM_ERROR(errno);
   }
   std::vector<std::string> v;
   char* line = nullptr;
   size_t len = 0;
   ssize_t read;
+  errno = 0;
+
+  OOMD_SCOPE_EXIT {
+    ::free(line);
+    ::fclose(fp);
+  };
   while ((read = ::getline(&line, &len, fp)) != -1) {
     OCHECK(line != nullptr);
     if (read > 0 && line[read - 1] == '\n') {
@@ -223,47 +298,49 @@ std::vector<std::string> Fs::readFileByLine(Fd&& fd) {
       v.emplace_back(line);
     }
   }
-  ::free(line);
-  ::fclose(fp);
+
+  // Error when reading file, and thus content might be corrupted
+  if (errno) {
+    return SYSTEM_ERROR(errno);
+  }
   return v;
 }
 
-std::vector<std::string> Fs::readControllers(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kControllersFile);
-  if (!lines.size()) {
-    return {};
+SystemMaybe<Unit> Fs::checkExistAt(const DirFd& dirfd, const char* name) {
+  if (int rc = ::faccessat(dirfd.fd(), name, F_OK, 0); rc == 0) {
+    return noSystemError();
+  } else {
+    return SYSTEM_ERROR(rc);
   }
-  return Util::split(lines[0], ' ');
 }
 
-std::vector<std::string> Fs::readControllersAt(const DirFd& dirfd) {
+SystemMaybe<std::vector<std::string>> Fs::readControllersAt(
+    const DirFd& dirfd) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kControllersFile));
-  if (!lines.size()) {
-    return {};
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
   }
-  return Util::split(lines[0], ' ');
+  return Util::split((*lines)[0], ' ');
 }
 
-std::vector<int> Fs::getPids(const std::string& path) {
-  std::vector<int> pids;
-  auto str_pids = readFileByLine(path + "/" + kProcsFile);
-  for (const auto& sp : str_pids) {
-    pids.push_back(std::stoi(sp));
-  }
-  return pids;
-}
-
-std::vector<int> Fs::getPidsAt(const DirFd& dirfd) {
-  std::vector<int> pids;
+SystemMaybe<std::vector<int>> Fs::getPidsAt(const DirFd& dirfd) {
   auto str_pids = readFileByLine(Fs::Fd::openat(dirfd, kProcsFile));
-  for (const auto& sp : str_pids) {
+  if (!str_pids) {
+    return SYSTEM_ERROR(str_pids.error());
+  }
+  std::vector<int> pids;
+  for (const auto& sp : *str_pids) {
     pids.push_back(std::stoi(sp));
   }
   return pids;
 }
 
-bool Fs::readIsPopulatedFromLines(const std::vector<std::string>& lines) {
-  for (const auto& line : lines) {
+SystemMaybe<bool> Fs::readIsPopulatedAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kEventsFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  for (const auto& line : *lines) {
     std::vector<std::string> toks = Util::split(line, ' ');
     if (toks.size() == 2 && toks[0] == "populated") {
       if (toks[1] == "1") {
@@ -271,22 +348,12 @@ bool Fs::readIsPopulatedFromLines(const std::vector<std::string>& lines) {
       } else if (toks[1] == "0") {
         return false;
       } else {
-        throw bad_control_file("invalid format");
+        return SYSTEM_ERROR(EINVAL);
       }
     }
   }
 
-  throw bad_control_file("invalid format");
-}
-
-bool Fs::readIsPopulated(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kEventsFile);
-  return readIsPopulatedFromLines(lines);
-}
-
-bool Fs::readIsPopulatedAt(const DirFd& dirfd) {
-  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kEventsFile));
-  return readIsPopulatedFromLines(lines);
+  return SYSTEM_ERROR(EINVAL);
 }
 
 std::string Fs::pressureTypeToString(PressureType type) {
@@ -296,10 +363,10 @@ std::string Fs::pressureTypeToString(PressureType type) {
     case PressureType::FULL:
       return "full";
   }
-  throw std::runtime_error("Invalid PressureType. Code should not be reached");
+  __builtin_unreachable();
 }
 
-ResourcePressure Fs::readRespressureFromLines(
+SystemMaybe<ResourcePressure> Fs::readRespressureFromLines(
     const std::vector<std::string>& lines,
     PressureType type) {
   auto type_name = pressureTypeToString(type);
@@ -321,15 +388,25 @@ ResourcePressure Fs::readRespressureFromLines(
       // full avg10=0.22 avg60=0.16 avg300=1.08 total=58464525
       std::vector<std::string> toks =
           Util::split(lines[pressure_line_index], ' ');
-      OCHECK_EXCEPT(toks[0] == type_name, bad_control_file("invalid format"));
+      if (toks[0] != type_name) {
+        return SYSTEM_ERROR(EINVAL);
+      }
       std::vector<std::string> avg10 = Util::split(toks[1], '=');
-      OCHECK_EXCEPT(avg10[0] == "avg10", bad_control_file("invalid format"));
+      if (avg10[0] != "avg10") {
+        return SYSTEM_ERROR(EINVAL);
+      }
       std::vector<std::string> avg60 = Util::split(toks[2], '=');
-      OCHECK_EXCEPT(avg60[0] == "avg60", bad_control_file("invalid format"));
+      if (avg60[0] != "avg60") {
+        return SYSTEM_ERROR(EINVAL);
+      }
       std::vector<std::string> avg300 = Util::split(toks[3], '=');
-      OCHECK_EXCEPT(avg300[0] == "avg300", bad_control_file("invalid format"));
+      if (avg300[0] != "avg300") {
+        return SYSTEM_ERROR(EINVAL);
+      }
       std::vector<std::string> total = Util::split(toks[4], '=');
-      OCHECK_EXCEPT(total[0] == "total", bad_control_file("invalid format"));
+      if (total[0] != "total") {
+        return SYSTEM_ERROR(EINVAL);
+      }
 
       return ResourcePressure{
           std::stof(avg10[1]),
@@ -346,7 +423,9 @@ ResourcePressure Fs::readRespressureFromLines(
       // full 0.00 0.03 0.05
       std::vector<std::string> toks =
           Util::split(lines[pressure_line_index + 1], ' ');
-      OCHECK_EXCEPT(toks[0] == type_name, bad_control_file("invalid format"));
+      if (toks[0] != type_name) {
+        return SYSTEM_ERROR(EINVAL);
+      }
 
       return ResourcePressure{
           std::stof(toks[1]),
@@ -357,146 +436,172 @@ ResourcePressure Fs::readRespressureFromLines(
     }
     case PsiFormat::MISSING:
       // Missing the control file
-      throw bad_control_file("missing file");
+      return SYSTEM_ERROR(ENOENT);
     case PsiFormat::INVALID:
-      throw bad_control_file("invalid format");
+      return SYSTEM_ERROR(EINVAL);
   }
-
-  // To silence g++ compiler warning about enums
-  throw std::runtime_error("Not all enums handled");
+  __builtin_unreachable();
 }
 
-int64_t Fs::readRootMemcurrent() {
+SystemMaybe<int64_t> Fs::readRootMemcurrent() {
   auto meminfo = getMeminfo("/proc/meminfo");
-  return meminfo["MemTotal"] - meminfo["MemFree"];
-}
-
-int64_t Fs::readMemcurrent(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemCurrentFile);
-  OCHECK_EXCEPT(lines.size() == 1, bad_control_file(path + ": missing file"));
-  return static_cast<int64_t>(std::stoll(lines[0]));
-}
-
-int64_t Fs::readMemcurrentAt(const DirFd& dirfd) {
-  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemCurrentFile));
-  OCHECK_EXCEPT(lines.size() == 1, bad_control_file("missing file"));
-  return static_cast<int64_t>(std::stoll(lines[0]));
-}
-
-ResourcePressure Fs::readRootMempressure(PressureType type) {
-  try {
-    auto lines = readFileByLine("/proc/pressure/memory");
-    return readRespressureFromLines(lines, type);
-  } catch (const bad_control_file& e) {
-    auto lines = readFileByLine("/proc/mempressure");
-    return readRespressureFromLines(lines, type);
+  if (!meminfo) {
+    return SYSTEM_ERROR(meminfo.error());
   }
+
+  if (!meminfo->count("MemTotal") || !meminfo->count("MemFree")) {
+    return SYSTEM_ERROR(EINVAL);
+  }
+  return (*meminfo)["MemTotal"] - (*meminfo)["MemFree"];
 }
 
-ResourcePressure Fs::readMempressure(
-    const std::string& path,
+SystemMaybe<int64_t> Fs::readMemcurrentAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemCurrentFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  return static_cast<int64_t>(std::stoll((*lines)[0]));
+}
+
+SystemMaybe<ResourcePressure> Fs::readRootMempressure(PressureType type) {
+  auto lines = readFileByLine("/proc/pressure/memory");
+  if (!lines) {
+    lines = readFileByLine("/proc/mempressure");
+  }
+
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  return readRespressureFromLines(*lines, type);
+}
+
+SystemMaybe<ResourcePressure> Fs::readMempressureAt(
+    const DirFd& dirfd,
     PressureType type) {
-  auto lines = readFileByLine(path + "/" + kMemPressureFile);
-  return readRespressureFromLines(lines, type);
-}
-
-ResourcePressure Fs::readMempressureAt(const DirFd& dirfd, PressureType type) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemPressureFile));
-  return readRespressureFromLines(lines, type);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  return readRespressureFromLines(*lines, type);
 }
 
-int64_t Fs::readMinMaxLowHighFromLines(const std::vector<std::string>& lines) {
-  OCHECK_EXCEPT(lines.size() == 1, bad_control_file("missing file"));
+SystemMaybe<int64_t> Fs::readMinMaxLowHighFromLines(
+    const std::vector<std::string>& lines) {
+  if (lines.size() != 1) {
+    return SYSTEM_ERROR(EINVAL);
+  }
   if (lines[0] == "max") {
     return std::numeric_limits<int64_t>::max();
   }
   return static_cast<int64_t>(std::stoll(lines[0]));
 }
 
-int64_t Fs::readMemlow(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemLowFile);
-  return Fs::readMinMaxLowHighFromLines(lines);
-}
-
-int64_t Fs::readMemlowAt(const DirFd& dirfd) {
+SystemMaybe<int64_t> Fs::readMemlowAt(const DirFd& dirfd) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemLowFile));
-  return Fs::readMinMaxLowHighFromLines(lines);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  auto ret = Fs::readMinMaxLowHighFromLines(*lines);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return ret;
 }
 
-int64_t Fs::readMemhigh(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemHighFile);
-  return Fs::readMinMaxLowHighFromLines(lines);
-}
-
-int64_t Fs::readMemhighAt(const DirFd& dirfd) {
+SystemMaybe<int64_t> Fs::readMemhighAt(const DirFd& dirfd) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemHighFile));
-  return Fs::readMinMaxLowHighFromLines(lines);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  auto ret = Fs::readMinMaxLowHighFromLines(*lines);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return ret;
 }
 
-int64_t Fs::readMemmax(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemMaxFile);
-  return Fs::readMinMaxLowHighFromLines(lines);
-}
-
-int64_t Fs::readMemmaxAt(const DirFd& dirfd) {
+SystemMaybe<int64_t> Fs::readMemmaxAt(const DirFd& dirfd) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemMaxFile));
-  return Fs::readMinMaxLowHighFromLines(lines);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  auto ret = Fs::readMinMaxLowHighFromLines(*lines);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return ret;
 }
 
-int64_t Fs::readMemhightmpFromLines(const std::vector<std::string>& lines) {
-  OCHECK_EXCEPT(lines.size() == 1, bad_control_file("missing file"));
+SystemMaybe<int64_t> Fs::readMemhightmpFromLines(
+    const std::vector<std::string>& lines) {
+  if (lines.size() != 1) {
+    return SYSTEM_ERROR(ENOENT);
+  }
   auto tokens = Util::split(lines[0], ' ');
-  OCHECK_EXCEPT(tokens.size() == 2, bad_control_file("invalid format"));
+  if (tokens.size() != 2) {
+    return SYSTEM_ERROR(EINVAL);
+  }
   if (tokens[0] == "max") {
     return std::numeric_limits<int64_t>::max();
   }
   return static_cast<int64_t>(std::stoll(tokens[0]));
 }
 
-int64_t Fs::readMemhightmp(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemHighTmpFile);
-  return readMemhightmpFromLines(lines);
-}
-
-int64_t Fs::readMemhightmpAt(const DirFd& dirfd) {
+SystemMaybe<int64_t> Fs::readMemhightmpAt(const DirFd& dirfd) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemHighTmpFile));
-  return readMemhightmpFromLines(lines);
-}
-
-int64_t Fs::readMemmin(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemMinFile);
-  return Fs::readMinMaxLowHighFromLines(lines);
-}
-
-int64_t Fs::readMemminAt(const DirFd& dirfd) {
-  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemMinFile));
-  return Fs::readMinMaxLowHighFromLines(lines);
-}
-
-int64_t Fs::readSwapCurrent(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemSwapCurrentFile);
-
-  // The swap controller can be disabled via CONFIG_MEMCG_SWAP=n
-  if (lines.size() == 1) {
-    return static_cast<int64_t>(std::stoll(lines[0]));
-  } else {
-    return 0;
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
   }
+  auto ret = readMemhightmpFromLines(*lines);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return ret;
 }
 
-int64_t Fs::readSwapCurrentAt(const DirFd& dirfd) {
+SystemMaybe<int64_t> Fs::readMemminAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemMinFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  auto ret = Fs::readMinMaxLowHighFromLines(*lines);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return ret;
+}
+
+SystemMaybe<int64_t> Fs::readSwapCurrentAt(const DirFd& dirfd) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemSwapCurrentFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
   // The swap controller can be disabled via CONFIG_MEMCG_SWAP=n
-  return lines.size() == 1 ? static_cast<int64_t>(std::stoll(lines[0])) : 0;
+  return std::stoll((*lines)[0]);
 }
 
-std::unordered_map<std::string, int64_t> Fs::getVmstat(
+SystemMaybe<int64_t> Fs::readSwapMaxAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemSwapMaxFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  auto ret = Fs::readMinMaxLowHighFromLines(*lines);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return ret;
+}
+
+SystemMaybe<std::unordered_map<std::string, int64_t>> Fs::getVmstat(
     const std::string& path) {
   auto lines = readFileByLine(path);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
   std::unordered_map<std::string, int64_t> map;
   char space{' '};
 
-  for (auto& line : lines) {
+  for (auto& line : *lines) {
     std::stringstream ss(line);
     std::string item;
 
@@ -512,14 +617,17 @@ std::unordered_map<std::string, int64_t> Fs::getVmstat(
   return map;
 }
 
-std::unordered_map<std::string, int64_t> Fs::getMeminfo(
+SystemMaybe<std::unordered_map<std::string, int64_t>> Fs::getMeminfo(
     const std::string& path) {
   char name[256] = {0};
   uint64_t val;
   std::unordered_map<std::string, int64_t> map;
 
   auto lines = readFileByLine(path);
-  for (auto& line : lines) {
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  for (auto& line : *lines) {
     int ret =
         sscanf(line.c_str(), "%255[^:]:%*[ \t]%" SCNu64 "%*s\n", name, &val);
     if (ret == 2) {
@@ -546,39 +654,44 @@ std::unordered_map<std::string, int64_t> Fs::getMemstatLikeFromLines(
   return map;
 }
 
-std::unordered_map<std::string, int64_t> Fs::getMemstat(
-    const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kMemStatFile);
-  return getMemstatLikeFromLines(lines);
-}
-
-std::unordered_map<std::string, int64_t> Fs::getMemstatAt(const DirFd& dirfd) {
+SystemMaybe<std::unordered_map<std::string, int64_t>> Fs::getMemstatAt(
+    const DirFd& dirfd) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemStatFile));
-  return getMemstatLikeFromLines(lines);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+
+  return getMemstatLikeFromLines(lines.value());
 }
 
-ResourcePressure Fs::readRootIopressure(PressureType type) {
+SystemMaybe<ResourcePressure> Fs::readRootIopressure(PressureType type) {
   auto lines = readFileByLine("/proc/pressure/io");
-  return readRespressureFromLines(lines, type);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  return readRespressureFromLines(*lines, type);
 }
 
-ResourcePressure Fs::readIopressure(
-    const std::string& path,
+SystemMaybe<ResourcePressure> Fs::readIopressureAt(
+    const DirFd& dirfd,
     PressureType type) {
-  auto lines = readFileByLine(path + "/" + kIoPressureFile);
-  return readRespressureFromLines(lines, type);
-}
-
-ResourcePressure Fs::readIopressureAt(const DirFd& dirfd, PressureType type) {
   auto lines = readFileByLine(Fs::Fd::openat(dirfd, kIoPressureFile));
-  return readRespressureFromLines(lines, type);
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  return readRespressureFromLines(*lines, type);
 }
 
-IOStat Fs::readIostatFromLines(const std::vector<std::string>& lines) {
-  std::vector<DeviceIOStat> io_stat;
-  io_stat.reserve(lines.size());
+SystemMaybe<IOStat> Fs::readIostatAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kIoStatFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
 
-  for (const auto& line : lines) {
+  std::vector<DeviceIOStat> io_stat;
+  io_stat.reserve(lines->size());
+
+  for (const auto& line : *lines) {
     // format
     //
     // 0:0 rbytes=0 wbytes=0 rios=0 wios=0 dbytes=0 dios=0
@@ -597,118 +710,165 @@ IOStat Fs::readIostatFromLines(const std::vector<std::string>& lines) {
         &dev_io_stat.dbytes,
         &dev_io_stat.dios);
 
-    OCHECK_EXCEPT(ret == 8, bad_control_file("invalid format"));
+    if (ret != 8) {
+      return SYSTEM_ERROR(EINVAL);
+    }
     dev_io_stat.dev_id = std::to_string(major) + ":" + std::to_string(minor);
     io_stat.push_back(dev_io_stat);
   }
   return io_stat;
 }
 
-IOStat Fs::readIostat(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kIoStatFile);
-  return readIostatFromLines(lines);
-}
-
-IOStat Fs::readIostatAt(const DirFd& dirfd) {
-  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kIoStatFile));
-  return readIostatFromLines(lines);
-}
-
-void Fs::writeControlFileAt(Fd&& fd, const std::string& content) {
-  char buf[1024];
-  buf[0] = '\0';
-  if (!fd.isValid()) {
-    throw bad_control_file(
-        std::string{"open failed: "} + ::strerror_r(errno, buf, sizeof(buf)));
+SystemMaybe<Unit> Fs::writeControlFileAt(
+    SystemMaybe<Fd>&& fd,
+    const std::string& content) {
+  if (!fd) {
+    return SYSTEM_ERROR(fd.error());
   }
-  auto ret = Util::writeFull(fd.fd(), content.c_str(), content.size());
+  auto ret = Util::writeFull(fd->fd(), content.c_str(), content.size());
   if (ret < 0) {
-    throw bad_control_file(
-        std::string{"write failed: "} + ::strerror_r(errno, buf, sizeof(buf)));
+    return SYSTEM_ERROR(errno);
   }
+
+  return noSystemError();
 }
 
-void Fs::writeMemhigh(const std::string& path, int64_t value) {
+SystemMaybe<Unit> Fs::writeMemhighAt(const DirFd& dirfd, int64_t value) {
   auto val_str = std::to_string(value);
-  writeControlFileAt(Fs::Fd::open(path + "/" + kMemHighFile, false), val_str);
+  auto ret =
+      writeControlFileAt(Fs::Fd::openat(dirfd, kMemHighFile, false), val_str);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return noSystemError();
 }
 
-void Fs::writeMemhighAt(const DirFd& dirfd, int64_t value) {
-  auto val_str = std::to_string(value);
-  writeControlFileAt(Fs::Fd::openat(dirfd, kMemHighFile, false), val_str);
-}
-
-void Fs::writeMemhightmp(
-    const std::string& path,
-    int64_t value,
-    std::chrono::microseconds duration) {
-  auto val_str = std::to_string(value) + " " + std::to_string(duration.count());
-  writeControlFileAt(
-      Fs::Fd::open(path + "/" + kMemHighTmpFile, false), val_str);
-}
-
-void Fs::writeMemhightmpAt(
+SystemMaybe<Unit> Fs::writeMemhightmpAt(
     const DirFd& dirfd,
     int64_t value,
     std::chrono::microseconds duration) {
   auto val_str = std::to_string(value) + " " + std::to_string(duration.count());
-  writeControlFileAt(Fs::Fd::openat(dirfd, kMemHighTmpFile, false), val_str);
-}
-
-int64_t Fs::getNrDyingDescendants(const std::string& path) {
-  auto lines = readFileByLine(path + "/" + kCgroupStatFile);
-  auto map = getMemstatLikeFromLines(lines);
-  // Will return 0 for missing entries
-  return map["nr_dying_descendants"];
-}
-
-int64_t Fs::getNrDyingDescendantsAt(const DirFd& dirfd) {
-  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kCgroupStatFile));
-  auto map = getMemstatLikeFromLines(lines);
-  // Will return 0 for missing entries
-  return map["nr_dying_descendants"];
-}
-
-KillPreference Fs::readKillPreferenceAt(const DirFd& path) {
-  if (Fs::hasxattrAt(path, kOomdPreferXAttr)) {
-    return KillPreference::PREFER;
-  } else if (Fs::hasxattrAt(path, kOomdAvoidXAttr)) {
-    return KillPreference::AVOID;
-  } else {
-    return KillPreference::NORMAL;
+  auto ret = writeControlFileAt(
+      Fs::Fd::openat(dirfd, kMemHighTmpFile, false), val_str);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
   }
+  return noSystemError();
 }
 
-bool Fs::setxattr(
+SystemMaybe<Unit> Fs::writeMemReclaimAt(const DirFd& dirfd, int64_t value) {
+  auto val_str = std::to_string(value);
+  auto ret = writeControlFileAt(
+      Fs::Fd::openat(dirfd, kMemReclaimFile, false), val_str);
+  if (!ret) {
+    return SYSTEM_ERROR(ret.error());
+  }
+  return noSystemError();
+}
+
+SystemMaybe<int64_t> Fs::getNrDyingDescendantsAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kCgroupStatFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  auto map = getMemstatLikeFromLines(lines.value());
+  // Will return 0 for missing entries
+  return map["nr_dying_descendants"];
+}
+
+SystemMaybe<KillPreference> Fs::readKillPreferenceAt(const DirFd& path) {
+  auto maybe = Fs::hasxattrAt(path, kOomdSystemPreferXAttr);
+  if (!maybe) {
+    return SYSTEM_ERROR(maybe.error());
+  }
+
+  if (*maybe) {
+    return KillPreference::PREFER;
+  }
+
+  maybe = Fs::hasxattrAt(path, kOomdUserPreferXAttr);
+  if (!maybe) {
+    return SYSTEM_ERROR(maybe.error());
+  }
+
+  if (*maybe) {
+    return KillPreference::PREFER;
+  }
+
+  maybe = Fs::hasxattrAt(path, kOomdSystemAvoidXAttr);
+  if (!maybe) {
+    return SYSTEM_ERROR(maybe.error());
+  }
+
+  if (*maybe) {
+    return KillPreference::AVOID;
+  }
+
+  maybe = Fs::hasxattrAt(path, kOomdUserAvoidXAttr);
+  if (!maybe) {
+    return SYSTEM_ERROR(maybe.error());
+  }
+
+  if (*maybe) {
+    return KillPreference::AVOID;
+  }
+
+  return KillPreference::NORMAL;
+}
+
+SystemMaybe<bool> Fs::readMemoryOomGroupAt(const DirFd& dirfd) {
+  auto lines = readFileByLine(Fs::Fd::openat(dirfd, kMemOomGroupFile));
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  return *lines == std::vector<std::string>({"1"});
+}
+
+SystemMaybe<Unit> Fs::setxattr(
     const std::string& path,
     const std::string& attr,
     const std::string& val) {
   int ret = ::setxattr(path.c_str(), attr.c_str(), val.c_str(), val.size(), 0);
   if (ret == -1) {
-    return false;
+    return SYSTEM_ERROR(errno);
   }
-  return true;
+  return noSystemError();
 }
 
-std::string Fs::getxattr(const std::string& path, const std::string& attr) {
+SystemMaybe<std::string> Fs::getxattr(
+    const std::string& path,
+    const std::string& attr) {
   std::string val;
 
   int size = ::getxattr(path.c_str(), attr.c_str(), nullptr, 0);
-  if (size <= 0) {
-    return val;
+  if (size == -1) {
+    if (errno == ENODATA) {
+      return val;
+    }
+    return SYSTEM_ERROR(errno);
   }
 
   val.resize(size);
-  ::getxattr(path.c_str(), attr.c_str(), &val[0], val.size());
+  size = ::getxattr(path.c_str(), attr.c_str(), &val[0], val.size());
+  if (size == -1) {
+    // We checked above but this could have raced, so check again
+    if (errno == ENODATA) {
+      return std::string{};
+    }
+    return SYSTEM_ERROR(errno);
+  }
   return val;
 }
 
-bool Fs::hasxattr(const std::string& path, const std::string& attr) {
-  return ::getxattr(path.c_str(), attr.c_str(), nullptr, 0) >= 0;
-}
-
-bool Fs::hasxattrAt(const DirFd& dirfd, const std::string& attr) {
-  return ::fgetxattr(dirfd.fd(), attr.c_str(), nullptr, 0) >= 0;
+SystemMaybe<bool> Fs::hasxattrAt(const DirFd& dirfd, const std::string& attr) {
+  auto ret = ::fgetxattr(dirfd.fd(), attr.c_str(), nullptr, 0);
+  if (ret == -1) {
+    if (errno == ENODATA || errno == EOPNOTSUPP) {
+      return false;
+    }
+    return SYSTEM_ERROR(errno);
+  }
+  return true;
 }
 
 bool Fs::isUnderParentPath(
@@ -735,9 +895,12 @@ bool Fs::isUnderParentPath(
   return true;
 }
 
-std::string Fs::getCgroup2MountPoint(const std::string& path) {
+SystemMaybe<std::string> Fs::getCgroup2MountPoint(const std::string& path) {
   auto lines = readFileByLine(path);
-  for (auto& line : lines) {
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  for (auto& line : *lines) {
     auto parts = Util::split(line, ' ');
     if (parts.size() > 2) {
       if (parts[2] == "cgroup2") {
@@ -745,23 +908,41 @@ std::string Fs::getCgroup2MountPoint(const std::string& path) {
       }
     }
   }
-  return "";
+  return SYSTEM_ERROR(EINVAL, path);
 }
 
-DeviceType Fs::getDeviceType(
+SystemMaybe<DeviceType> Fs::getDeviceType(
     const std::string& dev_id,
     const std::string& path) {
   const auto deviceTypeFile =
       path + "/" + dev_id + "/" + kDeviceTypeDir + "/" + kDeviceTypeFile;
   auto lines = readFileByLine(deviceTypeFile);
-  if (lines.size() == 1) {
-    if (lines[0] == "1") {
+  if (!lines) {
+    return SYSTEM_ERROR(lines.error());
+  }
+  if (lines->size() == 1) {
+    if ((*lines)[0] == "1") {
       return DeviceType::HDD;
-    } else if (lines[0] == "0") {
+    } else if ((*lines)[0] == "0") {
       return DeviceType::SSD;
     }
   }
-  throw bad_control_file(deviceTypeFile + ": invalid format");
+  return SYSTEM_ERROR(EINVAL, deviceTypeFile);
+}
+
+SystemMaybe<int> Fs::getSwappiness(const std::string& path) {
+  auto str_swappiness = readFileByLine(Fd::open(path));
+  if (!str_swappiness) {
+    return SYSTEM_ERROR(str_swappiness.error());
+  }
+  if (str_swappiness->size() != 1) {
+    return SYSTEM_ERROR(EINVAL, path, " malformed");
+  }
+  return std::stoi((*str_swappiness)[0]);
+}
+
+SystemMaybe<Unit> Fs::setSwappiness(int swappiness, const std::string& path) {
+  return writeControlFileAt(Fd::open(path, false), std::to_string(swappiness));
 }
 
 } // namespace Oomd
