@@ -20,6 +20,9 @@
 #include <unistd.h>
 
 #include <algorithm>
+#include <array>
+#include <cstring>
+#include <random>
 #include <sstream>
 
 static constexpr auto kWhitespaceChars = " \t\n\r";
@@ -201,4 +204,33 @@ ssize_t Util::writeFull(int fd, const char* msg_buf, size_t count) {
   return wrapFull(::write, fd, msg_buf, count);
 }
 
+std::string Util::generateUuid() {
+  unsigned int seed;
+  try {
+    seed = std::random_device{}();
+  } catch (const std::runtime_error&) {
+    seed = std::random_device("/dev/urandom")();
+  }
+  static std::mt19937 gen(seed);
+  static std::uniform_int_distribution<uint64_t> dis;
+
+  // Combine two 64-bit numbers
+  std::stringstream ss;
+  ss << std::hex << dis(gen) << dis(gen);
+
+  return ss.str();
+}
+
+std::string Util::strerror_r() {
+  std::array<char, 1024> buf;
+  buf[0] = '\0';
+
+  int savedErrno = errno;
+
+  std::string ret(::strerror_r(errno, buf.data(), buf.size()));
+
+  errno = savedErrno;
+  return ret;
+}
+
 } // namespace Oomd
