@@ -17,6 +17,7 @@
 
 #pragma once
 
+#include <algorithm>
 #include <cstdint>
 #include <string>
 #include <vector>
@@ -68,6 +69,17 @@ class Util {
   /* Trim spaces from a string */
   static void trim(std::string& s);
 
+  template <class T, class Functor>
+  static std::vector<T> filter(std::vector<T> elems, Functor&& fn) {
+    std::vector<T> ret;
+    std::copy_if(
+        elems.begin(),
+        elems.end(),
+        std::back_inserter(ret),
+        std::forward<Functor>(fn));
+    return ret;
+  }
+
   /*
    * Read and write helpers
    *
@@ -80,6 +92,13 @@ class Util {
    */
   static ssize_t readFull(int fd, char* msg_buf, size_t count);
   static ssize_t writeFull(int fd, const char* msg_buf, size_t count);
+
+  /*
+   * Returned uuids are random 32 char hex strings
+   */
+  static std::string generateUuid();
+
+  static std::string strerror_r();
 };
 
 } // namespace Oomd
